/* KBDispatcher provides a mechanism of multiplexing keyboard input to
 * different handlers, using a focus mechanism. The keyboard events to
 * be multiplexed must be told to the dispatcher using either
 * kbdispatcherTell or kbdispatcherTellKeyboardHandler (A function
 * suitable as a handler to give to kblib (see kblib.h), but otherwize
 * equivalent). */

#ifndef __KBDISPATCHLIB_H__
#define __KBDISPATCHLIB_H__
#include <ws/kblib.h>
#include <iAVLTree.h>


typedef void (*KBDispatchHandler)(long item, long input, void* data);
typedef void (*KBDispatchActivateHandler)(long item, void* data);

typedef struct
 {
  long                       index;
  KBDispatchHandler          handler;
  KBDispatchActivateHandler  activateHandler;
  KBDispatchActivateHandler  deactivateHandler;
  void                      *handlerData;
 } KBDispatcherEntry;

typedef struct
 {
  Key nextCmd;
  Key prevCmd;
  long next;
  KBDispatcherEntry *activeEntry;
  iAVLCursor active;
  iAVLTree *items;
 } KBDispatcher;

extern void kbdispatcherCreat              (KBDispatcher *dispatcher, Key prev, Key next);
extern void kbdispatcherDestroy            (KBDispatcher *dispatcher);
extern long kbdispatcherAdd                (KBDispatcher *dispatcher, KBDispatchHandler handler, KBDispatchActivateHandler activateHandler, KBDispatchActivateHandler deactivateHandler, void *handlerData);
extern void kbdispatcherRemove             (KBDispatcher *dispatcher, long item);
extern void kbdispatcherTell               (KBDispatcher *dispatcher, Key input);
extern void kbdispatcherTellKeyboardHandler(Key key, void *data);

#endif /* __KBDISPATCHLIB_H__ */
