/* Mainlooplib provides a mainloop, polling several filedescriptors
 * and calling callbacks when data becomes available/write becomes
 * possible, etc. The loop can be exited with a return value from
 * within a callback. */

#ifndef __MAINLOOPLIB_H__
#define __MAINLOOPLIB_H__

#include <sys/poll.h>
#include <ws/misc.h>
#include <ws/rhstack.h>
#include <ws/stacktypes.h>


/* Origin should be set to the function in which the error
 * occured. Error may be set to anything the failing function feels
 * like setting it to. */
typedef struct
 {
  void (*origin)(void);
  char *originName;
  void *error;
 } EventHandlerError;

struct MainLoopT;
typedef void (*EventHandler)(struct MainLoopT *mainloop, int fd, short events, EventHandlerError *data);
typedef struct pollfd Pollfd;
#define PollfdDelete(x)
#define PollfdCopy(x) x
STACKDECL(Pollfd)
#define EventHandlerDelete(x)
#define EventHandlerCopy(x) x
STACKDECL(EventHandler)
struct MainLoopT
 {
  Boolean            exit;
  Boolean            reset;
  EventHandlerError *exitData;
  PollfdStack        pollfds;
  EventHandlerStack  handlers;
  VoidPointerStack   handlerData;
 };
typedef struct MainLoopT MainLoop;

extern void               mainLoopCreat(MainLoop *mainloop);
extern void               mainLoopDestroy(MainLoop *mainloop);
extern EventHandlerError *mainLoopDo(MainLoop *mainloop);
extern void               mainLoopAdd(MainLoop *mainloop, int fd, short events, EventHandler handler, void *handlerData);
extern void               mainLoopRemove(MainLoop *mainloop, int fd, short events, EventHandler handler, void *handlerData);
extern void               mainLoopReset(MainLoop *mainloop);
extern void               mainLoopExit(MainLoop *mainloop, EventHandlerError *data);

#endif /*__MAINLOOPLIB_H__*/
