/* This library dispatches mouse events to several callback-functions,
 * based on rectangular regions and mouse-button-masks. */

#ifndef __MDISPATCHLIB_H__
#define __MDISPATCHLIB_H__

#include <ws/mouselib.h>
#include <iAVLTree.h>


typedef void (*MDispatchHandler)(Coordinate x, Coordinate y, MouseButtons buttons, void* data);
typedef void (*MDispatchExitHandler)(void* data);

typedef struct
 {
  long                  index;
  Coordinate            x;
  Coordinate            y;
  Size                  w;
  Size                  h;
  MouseButtons          buttons;
  MouseButtons          mask;
  Boolean               handled;
  MDispatchHandler      handler;
  MDispatchExitHandler  exitHandler;
  MDispatchExitHandler  changeHandler;
  void                 *handlerData;
 } MDispatcherEntry;

typedef struct
 {
  long          next;
  iAVLTree     *items;
 } MDispatcher;


extern void              mdispatcherCreat           (MDispatcher *dispatcher);
extern void              mdispatcherDestroy         (MDispatcher *dispatcher);
extern MDispatcherEntry *mdispatcherAdd             (MDispatcher *dispatcher, Coordinate x, Coordinate y, Size w, Size h, MouseButtons buttons, MouseButtons mask, MDispatchHandler handler, MDispatchExitHandler exitHandler, MDispatchExitHandler changeHandler, void *handlerData);
extern void              mdispatcherRemove          (MDispatcher *dispatcher, MDispatcherEntry *item);
extern void              mdispatcherTell            (MDispatcher *dispatcher, Coordinate x, Coordinate y, MouseButtons buttons);
extern void              mdispatcherTellMouseHandler(Coordinate position[MOUSE_DIMENSIONS], MouseButtons buttons, void *data);

#endif /* __MDISPATCHLIB_H__ */
