/* RedHog stacks are simple stacks of any datatype implemented as
 * realloc:ing arrays of the type specified at their creation. To
 * create a stack of specific type, the type must have a name (that
 * is, char * is not okay, but typedef char *foo; and then foo, is),
 * STACKDECL(typename) must be placed in some header-file, and
 * STACKDEF(typename) in one of the source-files. */

#ifndef __RHSTACK_H__
#define __RHSTACK_H__

#include <stdlib.h>


/* The stack definitions
 */

#define STACKDECL(type) \
typedef struct\
 {\
  type              *stack;\
  size_t             size;\
  size_t             usage;\
 } type##Stack;\
\
type##Stack type##StackCreate(void);\
void        type##StackDelete(type##Stack *stack);\
type##Stack type##StackCopy  (type##Stack  stack);\
typedef void (*type##StackMapFun)(type source, type##Stack *destination, void *arg);\
void        type##StackMap   (type##StackMapFun mapfun, type##Stack *source, type##Stack *destination, void *arg);\
size_t      type##StackPushf (type##Stack *stack, type obj);\
type        type##StackPopf  (type##Stack *stack);

#define STACKDEF(type) \
type##Stack type##StackCreate(void)\
 {\
  type##Stack stack;\
  \
  stack.stack = NULL;\
  stack.size = 0;\
  stack.usage = 0;\
  return stack;\
 }\
void type##StackDelete(type##Stack *stack)\
 {\
  for (;stack->usage > 0; stack->usage--)\
   type##Delete(&stack->stack[stack->usage - 1]);\
  free(stack->stack);\
  stack->stack = NULL;\
  stack->size = 0;\
 }\
\
type##Stack type##StackCopy(type##Stack stack)\
 {\
  type##Stack result;\
  \
  result.size = stack.size;\
  result.stack = (type *) malloc(sizeof(type) * result.size);\
  for (result.usage = 0; result.usage < stack.usage; result.usage++)\
   result.stack[result.usage] = type##Copy(stack.stack[result.usage]);\
  return result;\
 }\
\
void type##StackMap(type##StackMapFun mapfun, type##Stack *source, type##Stack *destination, void *arg)\
 {\
  size_t index;\
  \
  for (index = 0; index < source->usage; index++)\
   mapfun(source->stack[index], destination, arg);\
 }\
\
size_t type##StackPushf(type##Stack *stack, type obj)\
 {\
  if (stack->usage >= stack->size)\
   stack->stack = (type *) realloc(stack->stack, sizeof(type) * (stack->size += 16));\
  stack->stack[stack->usage] = obj;\
  return stack->usage++; \
 }\
type type##StackPopf(type##Stack *stack)\
 {\
  return stack->stack[--stack->usage];\
 }

#endif /* __RHSTACK_H__ */
